// Copyright 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

syntax = "proto2";

option optimize_for = LITE_RUNTIME;

package extensions.api.cast_channel.proto;

message AuthorityKeys {
  message Key {
    required bytes fingerprint = 1;
    required bytes public_key = 2;
  }
  repeated Key keys = 1;
}
